import { Request, Response } from 'express'

import { RequestContext } from '@/libs/domain/contexts/request.context'
import { AdminSignOutUseCase } from '@/libs/application/use-cases/admin-auth/sign-out-use-case'
import { AdminSignInUseCase } from '@/libs/application/use-cases/admin-auth/sign-in-use-case'
import { container } from '../di/container'

const signInUseCase = container.get(AdminSignInUseCase)
const signOutUseCase = container.get(AdminSignOutUseCase)

export class AdminAuthController {
  static async signIn(req: Request, res: Response) {
    const tokens = await signInUseCase.run(req.body)

    return res.json({ ...tokens })
  }

  static async signOut(req: Request, res: Response) {
    await signOutUseCase.run(RequestContext.fromPrimitives(req))

    return res.json({ message: 'Sesion finalizada con éxito.' })
  }
}
