import { NextFunction, Request, Response } from 'express'
import { GetMyProfileUseCase } from '@/libs/application/use-cases/user/get-my-profile.use-case'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { UnauthorizedException } from '@/libs/domain/errors'
import { container } from '../di/container'

const getMyProfileUseCase = container.get(GetMyProfileUseCase)

export const authMiddleware =
  (roles: string[] = []) =>
  async (req: Request, res: Response, next: NextFunction) => {
    try {
      const getToken = (header = '') => header.split(' ').pop()
      const token = getToken(req.header('authorization'))

      if (!token) {
        throw new UnauthorizedException()
      }

      const user = await getMyProfileUseCase.run(
        { token, roles },
        RequestContext.fromPrimitives(req)
      )

      req.user = user

      // @ts-ignore
      delete req.user?.password

      return next()
    } catch (error) {
      return next(error)
    }
  }
