import { authMiddleware } from '@/core/api/middlewares/auth.middleware'
import Router from 'express-promise-router'
import { CustomerController } from '../controllers/customer.controller'
import { ParametersValidationSchema } from '@/core/api/validation-schemas/parameters.validation-schema'
import { CustomerValidationSchema } from '../validation-schemas/customer.validation-schema'
import { validateUserEmailMiddleware } from '@/core/api/middlewares/validate-user-email.middleware'
import { validateUserRutMiddleware } from '@/core/api/middlewares/validate-user-rut.middleware'
import { Role } from '@/libs/infrastructure/persistance/seeders/dev.seeder'
import { validateIdExistsMiddleware } from '@/core/api/middlewares/validate-id-exists.middleware'

export const customerRouter = Router()
  .use(authMiddleware([Role.adminEmployee, Role.vendorEmployee]))
  .post(
    '/',
    CustomerValidationSchema.create,
    validateUserEmailMiddleware('user.email'),
    validateUserRutMiddleware('user.rut'),
    CustomerController.create
  )
  .use('/:id', validateIdExistsMiddleware('customer'))
  .patch('/:id', CustomerValidationSchema.update, CustomerController.update)
  .delete('/:id', CustomerController.delete)
  .get('/:id', CustomerController.show)
  .get('/', ParametersValidationSchema.offsetCriteria, CustomerController.list)
