import { Injectable } from '@/core/api/di/decorators'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { BadRequestException } from '@/libs/domain/errors'
import { EncryptHelper } from '@/libs/domain/helpers/encrypt.helper'
import { TokenHelper } from '@/libs/domain/helpers/token.helper'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'

@Injectable()
export class ResetPasswordUseCase {
  constructor(private readonly userRepository: UserRepository) {}

  async run(
    payload: { newPassword: string; token: string },
    { tenant }: RequestContext
  ): Promise<void> {
    const token = await TokenHelper.parseJWT(payload.token)
    if (!token) {
      throw new BadRequestException('Codigo invalido')
    }

    const user = await this.userRepository.findOne({ id: token.id }, tenant.id)
    if (!user) {
      throw new BadRequestException('Usuario no existe')
    }

    await this.userRepository.save(
      {
        ...user,
        password: await EncryptHelper.hash(payload.newPassword),
      },
      tenant.id
    )
  }
}
