import { Injectable } from '@/core/api/di/decorators'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { TokenEntity } from '@/libs/domain/entities/tenant/token.entity'
import { BadRequestException } from '@/libs/domain/errors'
import { EncryptHelper } from '@/libs/domain/helpers/encrypt.helper'
import { TokenHelper } from '@/libs/domain/helpers/token.helper'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'

@Injectable()
export class SignInUseCase {
  constructor(private readonly userRepository: UserRepository) {}
  async run(
    payload: { rut: string; password: string },
    { tenant }: RequestContext
  ): Promise<TokenEntity> {
    const user = await this.userRepository.findOne(
      {
        rut: payload.rut,
      },
      tenant.id
    )

    if (!user || !(await EncryptHelper.compare(payload.password, user.password))) {
      throw new BadRequestException('Credenciales incorrectas')
    }

    if (!user.isActive) {
      throw new BadRequestException('Usuario deshabilitado')
    }

    const accessToken = await TokenHelper.generateJWT({ id: user.id })
    const tokens: TokenEntity = {
      accessToken,
    }

    return tokens
  }
}
