import { Injectable } from '@/core/api/di/decorators'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { BadRequestException } from '@/libs/domain/errors'
import { DriverRepository } from '@/libs/domain/repositories/tenant/driver.repository'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'

@Injectable()
export class DeleteDriverUseCase {
  constructor(
    private readonly driverRepository: DriverRepository,
    private readonly userRepository: UserRepository
  ) {}

  async run(id: string, { tenant }: RequestContext): Promise<void> {
    const user = await this.userRepository.findOne(
      {
        driver: {
          id,
        },
      },
      tenant.id
    )

    if (!user) {
      throw new BadRequestException('No existe usuario relacionado.')
    }

    if (!user.employee && !user.customer) {
      await this.userRepository.delete(
        {
          id: user?.id,
        },
        tenant.id
      )
      return
    }

    await this.driverRepository.delete(
      {
        id,
      },
      tenant.id
    )
  }
}
