import { Injectable } from '@/core/api/di/decorators'
import { UserEntity } from '@/libs/domain/entities/tenant/user.entity'
import { ForbiddenException, UnauthorizedException } from '@/libs/domain/errors'
import { TokenHelper } from '@/libs/domain/helpers/token.helper'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'
import { RequestContext } from '../../../domain/contexts/request.context'

@Injectable()
export class GetMyProfileUseCase {
  constructor(private readonly userRepository: UserRepository) {}

  async run(
    { token, roles = [] }: { token: string; roles?: string[] },
    { tenant }: RequestContext
  ): Promise<UserEntity> {
    const payload = await TokenHelper.parseJWT(token ?? '')
    if (!payload?.id) {
      throw new UnauthorizedException()
    }
    const user = await this.userRepository.findOne(
      {
        id: payload?.id,
      },
      tenant.id
    )

    if (!user) {
      throw new UnauthorizedException()
    }

    // @ts-ignore
    delete user?.password

    if (roles?.length && !user.roles?.map(role => role.name).some(role => roles.includes(role!))) {
      throw new ForbiddenException()
    }

    return user
  }
}
