import { UserEntity } from '@/libs/domain/entities/tenant/user.entity'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'
import { EntitySchema } from 'typeorm'
import { userSchema } from '../../schemas/tenant/user.schema'
import {
  FindCriteriaOptions,
  FindOneOptions,
  FindOptions,
  Relations,
} from '../typeorm-criteria-parser'
import { TypeOrmBaseTenantRepository } from './_typeorm-base-tenant-repository'
import { Injectable } from '@/core/api/di/decorators'

@Injectable()
export class TypeOrmUserRepository
  extends TypeOrmBaseTenantRepository<UserEntity>
  implements UserRepository
{
  protected getSchema(): EntitySchema<UserEntity> {
    return userSchema
  }

  protected getRelations(): Relations<UserEntity> {
    return {
      roles: {
        permissions: true,
      },
      customer: true,
      driver: {
        transportCompany: true,
      },
      employee: true,
    }
  }

  protected getFindOneOptions(): FindOneOptions<UserEntity> {
    return {
      select: {
        id: true,
        rut: true,
        password: true,
        email: true,
        firstName: true,
        lastName: true,
        phoneNumber: true,
        isActive: true,
        createdAt: true,
        updatedAt: true,
        tenantId: true,
      },
    }
  }

  protected getFindOptions(): FindOptions<UserEntity> {
    return {}
  }

  protected getFindCriteriaOptions(): FindCriteriaOptions<UserEntity> {
    return {}
  }
}
