import { environment } from '@/core/api/environment'
import { Job, Queue, QueueEvents, QueueEventsListener, Worker } from 'bullmq'

export class BaseQueue {
  private name: string
  private connection = {
    host: environment.REDIS_HOST,
    port: Number(environment.REDIS_PORT),
  }

  private queue: Queue
  private events: QueueEvents
  private workers: Worker[] = []

  constructor(name: string) {
    this.name = name

    this.queue = new Queue(this.name, {
      connection: {
        ...this.connection,
      },

      defaultJobOptions: {
        removeOnComplete: process.env.NODE_ENV === 'production' ? 1000 : false,
        removeOnFail: 5000,
      },
    })

    this.events = new QueueEvents(this.name, { connection: this.connection })
  }

  addJob<DataType = any>(name: string, data: DataType) {
    this.queue.add(name, data)
  }

  addWorker<Data>(callback: (job: Job<Data>) => Promise<void>) {
    const worker = new Worker(this.name, callback, {
      connection: this.connection,
      concurrency: 10,
    })
    this.workers.push(worker)
    return worker
  }

  addEventListener<U extends keyof QueueEventsListener>(
    event: U,
    listener: QueueEventsListener[U]
  ) {
    return this.events.on(event, listener)
  }

  async close() {
    await this.queue.close()
    for (const worker of this.workers) {
      await worker.close()
    }
    this.events.removeAllListeners()
    await this.events.close()
  }
}
